import React, { Component } from 'react';
import { SnackbarHOC } from 'educoder';
import {BrowserRouter as Router,Route,Switch} from 'react-router-dom';
import {Tooltip,Menu,Pagination, Spin} from 'antd';
import Loadable from 'react-loadable';
import Loading from '../../../Loading';
import NoneData from '../../courses/coursesPublic/NoneData'
import axios from 'axios';
import {getImageUrl} from 'educoder';
import { TPMIndexHOC } from '../../tpm/TPMIndexHOC';
import { CNotificationHOC } from '../../courses/common/CNotificationHOC'
import "./usersInfo.css"

import Create from './publicCreatNew'

class InfosPath extends Component{
  constructor(props){
    super(props);
    this.state={
      category:undefined,
      page:1,
      sort_by:'time',
      status:undefined,
      per_page:16,
      isSpin:false,

      totalCount:undefined,
      data:undefined
    }
  }

  componentDidMount=()=>{
    this.setState({
      isSpin:true
    })
    let{category,status,sort_by,page,per_page}=this.state;
    this.getCourses(category,status,sort_by,page,per_page);
  }

  getCourses=(category,status,sort_by,page,per_page)=>{
    let url=`/users/${this.props.login}/subjects.json`;
    axios.get((url),{params:{
      category,
      status,
      sort_by,
      page,
      per_page:category && page ==1?17:16
    }}).then((result)=>{
      if(result){
        this.setState({
          totalCount:result.data.count,
          data:result.data,
          isSpin:false
        })
      }
    }).catch((error)=>{
      console.log(error);
    })
  }

  //切换种类
  changeCategory=(cate)=>{
    this.setState({
      category:cate,
      status:undefined,
      page:1,
      isSpin:true
    })
    let{sort_by}=this.state;
    this.getCourses(cate,undefined,sort_by,1);
  }
  // 切换状态
  changeStatus=(status)=>{
    let{category,sort_by}=this.state;
    this.setState({
      status,
      page:1,
      isSpin:true
    })
    this.getCourses(category,status,sort_by,1);
  }
  //切换页数
  changePage=(page)=>{
    this.setState({
      page,
      isSpin:true
    })
    let{category,sort_by,status}=this.state;
    this.getCourses(category,status,sort_by,page);
  }

  // 进入课堂
  turnToCourses=(url)=>{
    this.props.history.push(url);
  }


  // 切换排序方式
  changeOrder= (sort)=>{
    this.setState({
      sort_by:sort,
      isSpin:true
    })
    let{category,status,page}=this.state;
    this.getCourses(category,status,sort,page);
  }

  render(){
    let{
      category,
      status,
      sort_by,
      page,
      data,
      totalCount,
      isSpin
    } = this.state;
    let isStudent = this.props.isStudent();
    let  is_current=this.props.is_current;
    return(
      <div className="educontent">
        <Spin size="large" spinning={isSpin}>
        <div className="white-panel edu-back-white pt25 pb25 clearfix ">
          <li className={category ? "" : "active"}><a href="javascript:void(0)" onClick={()=>this.changeCategory()}>全部</a></li>
          <li className={category=="manage" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeCategory("manage")}>{is_current ? "我":"TA"}管理的</a></li>
          <li className={category=="study" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeCategory("study")}>{is_current ? "我":"TA"}学习的</a></li>
        </div>
        {
          category && category == "manage" &&
          <div className="edu-back-white padding20-30 clearfix secondNav bor-top-greyE">
            <li className={status ? "" : "active"}><a href="javascript:void(0)" onClick={()=>this.changeStatus()}>全部</a></li>
            <li className={status=="editing" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeStatus("editing")}>编辑中</a></li>
            <li className={status=="applying" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeStatus("applying")}>待审核</a></li>
            <li className={status=="published" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeStatus("published")}>已发布</a></li>
          </div>
        }
        {
          category && category == "study" &&
          <div className="edu-back-white padding20-30 clearfix secondNav bor-top-greyE">
            <li className={status ? "" : "active"}><a href="javascript:void(0)" onClick={()=>this.changeStatus()}>全部</a></li>
            <li className={status=="unfinished" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeStatus("unfinished")}>未完成</a></li>
            <li className={status=="finished" ? "active" : ""}><a href="javascript:void(0)" onClick={()=>this.changeStatus("finished")}>已完成</a></li>
          </div>
        }
        <div className="pl25 pr25 clearfix font-12 mb20 mt20">
          <span className="fl color-grey-9">共参与{totalCount}个{category?category=="manage"?"发布":"学习":"实训课程"}</span>
          <span className="fr color-grey-9">时间最新</span>
        </div>
        <div className="square-list clearfix">
          {
            isStudent ==false && page == 1 && !category &&
            <Create href={"/paths/new"} name={"新建实训课程"} index="3"></Create>
          }
          {
            (!data || data.subjects.length==0) && (isStudent || category) && <NoneData></NoneData>
          }
          {
            data && data.subjects && data.subjects.map((item,key)=>{
              return(
                <div className="square-Item" onClick={()=>this.turnToCourses(`/paths/${item.id}`)}>
                  {
                    item.tag && <div className="tag-green"><span className="tag-name">{item.tag}</span><img src={getImageUrl("/images/educoder/tag2.png")}/></div>
                  }                  
                  <a href="javascript:void(0)" className="square-img"><img alt="Subject12" src={getImageUrl(`${item.image_url}`)}/></a>
                  <div className="square-main">
                    <p className="task-hide">
                      <a href="javascript:void(0)" className="justify color-grey-name">{item.name}</a>
                    </p>
                    <div className="mt10">
                      <p className="color-grey-6 clearfix">
                        <a href="javascript:void(0)" className="fl color-grey-9">{item.owner_name}</a>
                        <span className="fr squareIconSpan">
                          <Tooltip placement='bottom' title="访问量">
                            <i className="iconfont icon-liulanyan fl mr5"></i>
                          </Tooltip>
                          {item.visits_count}
                        </span>
                      </p>
                    </div>
                  </div>
                </div>
              )
            })
          }
        </div>
        {
          totalCount > 15 && 
          <div className="mt30 mb50 edu-txt-center">
            <Pagination showQuickJumper total={totalCount} onChange={this.changePage} pageSize={16} current={page}/>
          </div>
        }
        </Spin>
      </div>
    )
  }
}
export default InfosPath;